@@ -129,7 +129,7 @@ for (@ifiles) {
       else {
          my $tmp=$_;
          $tmp=~s/^/\//;
-         $tmp=~s/_/\//;
+         $tmp=~s/_/\//g;
          &get($tmp);
       }
    }
