@@ -8,9 +8,8 @@
 #	remove obsolete directories or files and not touch required
 #	directories or files.
 #
-#	Usage: setup.pl cache-dir
 
-my $path = $ENV{PATH_INFO};
+umask 0022;
 
 #############################################################################
 ### configuration ###########################################################
@@ -43,27 +42,41 @@ my $private_dir = "$config{cache_dir}/private";
 # because cache_dir is preset to a default value prior to loading the config file)
 die "Warning: config file could not be parsed ($configfile)/ (cache_dir is not set)\n" if ($config{cache_dir} eq '');
 
-foreach my $file (qw/README README.txt/) {
-	print "Checking for $file ...\n";
-	if (-f $file) {
-		print "Unlinking $file (obsolete)\n";
-		unlink($file);
-	}
+
+@info=getpwnam("www-data");
+my @permcmd;
+if(-e $config{cache_dir}) {
+   @permcmd = ("chown", "--reference", $config{cache_dir});
+}
+elsif(@info) {
+   print "Assuming www-data is the user ID used to run apt-cacher\n";
+   @permcmd = ("chown", "$info[2]:$info[3]");
+}
+else {
+   @permcmd = ("/bin/echo", "User account for apt-cacher/http daemon unknown, plese set ownership for the following files manually:");
 }
 
-foreach my $dir ($config{cache_dir}, "$config{cache_dir}/private", "$config{cache_dir}/tmp", "$config{cache_dir}/import") {
+for ("README", "README.txt") {
+   my $file=$config{cache_dir}."/$_";
+   if (-f $file) {
+      print "Found obsolete file $file - removing.\n";
+      unlink($file);
+   }
+}
+
+foreach my $dir ($config{cache_dir}, $config{logdir}, "$config{cache_dir}/private", "$config{cache_dir}/import", "$config{cache_dir}/packages", "$config{cache_dir}/headers") {
 	if (!-d $dir) {
 		print "Doing mkdir($dir, 0755)\n";
 		mkdir($dir, 0755);
+    system (@permcmd, $dir);
 	}
 	if (!-w $dir) {
-		die "$dir is not writeable by you!\n";
+		die "Warning, $dir exists but is not is not writeable for apt-cacher!\n";
 	}
 }
 
 # Remove these directories if they exist (obsolete)
-# later rmdir $cache_dir/tmp, just not now as I may want it in future
-foreach my $rmdir ("$config{cache_dir}/head") {
+foreach my $rmdir ("$config{cache_dir}/tmp", "$config{cache_dir}/head") {
 	if (-d $rmdir) {
 		print "Doing 'rm -rf $rmdir' (obsolete)\n";
 		system("rm -rf $rmdir");
@@ -73,15 +86,18 @@ foreach my $rmdir ("$config{cache_dir}/head") {
 # At the moment we need to create empty access and error logs so apt-cacher
 # doesn't barf the first time it's run. Probably should change apt-cacher
 # so it can handle missing logs, and create them itself if required.
-`chown www-data.www-data $config{logdir}`;
-`touch $config{logfile}`;
-`chown www-data.www-data $config{logfile}`;
-`touch $config{errorfile}`;
-`chown www-data.www-data $config{errorfile}`;
+for $file ($config{logfile}, $config{errorfile}) {
+   if(!-e $file) {
+      open(my $tmp, ">$file");
+      close($tmp);
+      system @permcmd, $file;
+   }
+}
 
 # These ownership changes are a cludge: need to make them check httpd.conf for the Apache
 # user and set ownership to that, and do it with Perl instead of shell
-`chown -R www-data.www-data $config{cache_dir}`;
+# EB: fsck that, this may simply overwritte changes by the admin
+# `chown -R www-data.www-data $config{cache_dir}`;
 
 # We used to tack a line onto the end of apache.conf. Now we just symlink into conf.d
 if(-d "/etc/apache/conf.d" ){
@@ -93,7 +109,7 @@ if(-d "/etc/apache-ssl/conf.d" ){
 }
 
 if(-d "/etc/apache2/conf.d" ){
-	symlink("/etc/apt-cacher/apache.conf","/etc/apache2/conf.d/apt-cacher");
+	rename("/etc/apache2/conf.d/apt-cacher", "/etc/apache2/conf.d/apt-cacher.conf") || symlink("/etc/apt-cacher/apache.conf","/etc/apache2/conf.d/apt-cacher.conf");
 }
 
 # Apache2 needs the cgi module installed, which it isn't by default.
@@ -128,6 +144,10 @@ if (-f $httpdconf) {
 	
 	rename($old, $bak)          or die "can't rename $old to $bak: $!";
 	rename($new, $old)          or die "can't rename $new to $old: $!";
+	if (-f "/etc/init.d/apache")
+	{
+		`/etc/init.d/apache restart`;
+	}
 }
 
 # Remove the include lines from Apache-SSL's httpd.conf
@@ -151,6 +171,10 @@ if (-f $httpdconf) {
 	
 	rename($old, $bak)          or die "can't rename $old to $bak: $!";
 	rename($new, $old)          or die "can't rename $new to $old: $!";
+	if (-f "/etc/init.d/apache-ssl")
+	{
+		`/etc/init.d/apache-ssl restart`;
+	}
 }
 
 # Remove the include lines from Apache2's apache2.conf
@@ -174,6 +198,10 @@ if (-f $httpdconf) {
 
         rename($old, $bak)          or die "can't rename $old to $bak: $!";
         rename($new, $old)          or die "can't rename $new to $old: $!";
+	if (-f "/etc/init.d/apache2")
+	{
+		`/etc/init.d/apache2 restart`;
+	}
 }
 #^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 
